/* hazmap: v.1.0.0 */
/* encoding: UTF-8 & CR+LF */

////////////////////////////////////////////////////////////////////////
// 前提とするライブラリ等
//----------------------------------------------------------------------
// - Leaflet 0.7.3
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
// 参考としたソースコード
//----------------------------------------------------------------------
// gsimaps (地理院地図)（GitHub）
//    https://github.com/gsi-cyberjapan/gsimaps
//    commit: 020f5e47265b4c8b508b37ffd8acc025b308083e
//----------------------------------------------------------------------
// スタイルつき GeoJSON 規約（GitHub）
//    https://github.com/gsi-cyberjapan/geojson-with-style-spec
//    commit: fa8e35bf68f35905a34db21db05af147cb2bc9b6
////////////////////////////////////////////////////////////////////////

////////////////////////////////////////////////////////////////////////
// グローバル
////////////////////////////////////////////////////////////////////////

// マップオブジェクト
var map = null;

////////////////////////////////////////////////////////////////////////
// GSI
////////////////////////////////////////////////////////////////////////

var GSI = GSI || {};
GSI.Utils = GSI.Utils || {};

GSI.Utils.encodeHTML = function(src) {
	src = src.replace(/&/g, "&amp;");
	src = src.replace(/</g, "&lt;");
	src = src.replace(/>/g, "&gt;");
	return src;
};

GSI.DivIcon = L.DivIcon.extend( {
	options: {
		iconSize: null,
		className: "gsi-div-icon",
		html: false
	},
	createIcon: function(oldIcon) {
		var div = L.DivIcon.prototype.createIcon.call(this, oldIcon);
		return div;
	}
});

GSI.divIcon = function(options) {
	return new GSI.DivIcon(options);
};

////////////////////////////////////////////////////////////////////////
// CONFIG
////////////////////////////////////////////////////////////////////////

var CONFIG = CONFIG || {};

//
// GeoJSON 独自パラメータ
//----------------------------------------------------------------------
//

CONFIG.GEOJSONSPECIALKEYS = {
	"_markerType": true,

	"_iconUrl": true,
	"_iconSize": true,
	"_iconAnchor": true,
	"_html": true,
	"_radius": true,

	"_className": true,
	"_stroke": true,
	"_radius": true,

	"_dashArray": true,
	"_lineCap": true,
	"_lineJoin": true,
	"_clickable": true,

	"_color": true,
	"_opacity": true,
	"_weight": true,
	"_fill": true,
	"_fillColor": true,
	"_fillOpacity": true,
	"_weight": true
};

//
//  設定：作図
//----------------------------------------------------------------------
//

CONFIG.SAKUZU = {
	SYMBOL: {
		ICONSIZE : [20, 20]
	}
};

////////////////////////////////////////////////////////////////////////
// GSI.GeoJSON
////////////////////////////////////////////////////////////////////////

//
// _xxx 形式のアイコンのオプション設定を xxx 形式にして返す
//----------------------------------------------------------------------
//

function createIconOptions(properties, optionKeys) {
	var options = {};

	$.each(optionKeys, function(i, key) {
		var value = properties["_" + key];
		if (value) {
			options[key] = value;
		}
	});

	return options;
}

//
// _xxx 形式のマーカーのオプション設定を xxx 形式にして返す
//----------------------------------------------------------------------
//

function createMarkerOptions(properties) {
	var options = {};

	for (var key in properties) {
		if (!properties[key]) {
			continue;
		}

		if (key !== "" && key.charAt(0) === "_" && key !== "_markerType") {
			var value = properties[key];
			key = key.slice(1);
			options[key] = value;
		}
	}

	return options;
}

//
// デフォルトのマーカーを返す
//----------------------------------------------------------------------
//

function getDefaultMarker(latlng) {
	return L.marker(latlng, {
		icon: L.icon({
			iconUrl: "http://cyberjapandata.gsi.go.jp/portal/sys/v4/symbols/080.png",
			iconSize: [20, 20],
			iconAnchor: [10, 10]
		})
	});
}

//
// GeoJSON 形式のデータを読み込む L.geoJson() へのオプション
//----------------------------------------------------------------------
//

var geoJsonOptions = {

	// pointToLayer
	//----------------------------------------------------------------------

	pointToLayer: function (feature, latlng) {

		if (!feature.properties) {
			return getDefaultMarker(latlng);
		}

		var marker = null;

		if (feature.properties["_markerType"]) {

			var markerType = feature.properties["_markerType"];

			switch (markerType) {

			case "DivIcon":
				var options = createIconOptions(feature.properties, ["iconSize", "iconAnchor", "html", "className"]);
				marker = L.marker(latlng, {
										icon: GSI.divIcon(options)
									});
				break;

			case "CircleMarker":
				var options = createMarkerOptions(feature.properties);
				marker = L.circleMarker(latlng, options);
				marker.setRadius(options["radius"]);
				break;

			case "Circle":
				var options = createMarkerOptions(feature.properties);
				marker = L.circle(latlng, options["radius"],options);
				break;
			}
		}

		if (!marker) {
			if (!feature.properties["_iconUrl"]) {
				return getDefaultMarker(latlng);
			}

			var options = createIconOptions(feature.properties, ["iconUrl", "iconSize", "iconAnchor", "className"]);
			if (options.iconSize) {
				options._iconScale = options.iconSize[0] / CONFIG.SAKUZU.SYMBOL.ICONSIZE[0];
			}

			marker = L.marker(latlng, {
									icon: L.icon(options)
								});
		}

		return marker;
	},

	// style
	//----------------------------------------------------------------------

	style: function (feature) {

		if (!feature.properties) {
			return null;
		}

		var style = null;
		var iconStyleKeys = {
			"_markerType": true,
			"_iconUrl": true,
			"_iconSize": true,
			"_iconAnchor": true,
			"_html": true,
			"_radius": true
		};

		for (var key in feature.properties) {
			if (!feature.properties[key] && feature.properties[key] !== 0 ) {
				continue;
			}

			if (key !== "" && key.charAt(0) === "_") {
				if (iconStyleKeys[key]) {
					continue;
				} else {
					var value = feature.properties[key];
					key = key.slice(1);
					if (!style) { style = {}; }
					style[key] = value;
				}
			}
		}
		return style;
	},

	// onEachFeature
	//----------------------------------------------------------------------

	onEachFeature: function (feature, layer) {

		if (!feature.properties) {
			return;
		}

		var popupContent = "";

		if (feature.properties["name"]) {
			popupContent += "<h2>" + GSI.Utils.encodeHTML(feature.properties["name"]) + "</h2>";
		}

		if (feature.properties["description"]) {

			popupContent += feature.properties["description" ];

		} else {

			var table = "";

			for (var key in feature.properties) {

				if (!feature.properties[key]) {
					continue;
				}

				if (key !== "" && key !== "name" && !CONFIG.GEOJSONSPECIALKEYS[key]) {
					table +=
						"<tr>" +
						"<td>" + key + "</td>" +
						"<td>" + feature.properties[key] + "</td>" +
						"</tr>";
				}
			}

			if (table !== "") {
				table = "<table>" + table + "</table>";
				popupContent += table;
			}
		}

		if (popupContent !== "") {
			layer.bindPopup(popupContent, {
					maxWidth: 5000
				});
		}
	}

};

////////////////////////////////////////////////////////////////////////
// マップの作成とレイヤのロードを行う
////////////////////////////////////////////////////////////////////////

//
// エラー表示を行う
//----------------------------------------------------------------------
//

function displayError(message) {
	$("#loadErrorMessage").html(message);
	console.log(message);
}

//
// すべてのタイルレイヤを作成する
//----------------------------------------------------------------------
//

function createAllTileLayer() {

	var tileLayers = [];

	for (var i = 0; i < HZ_mapTiles.length; ++i) {
		var tile = HZ_mapTiles[i];

		var tileLayer = L.tileLayer(tile.urlTemplate, {
						attribution: "<a href='http://maps.gsi.go.jp/development/ichiran.html' target='_blank'>地理院タイル（" + tile.name + "）</a>"
					});

		tileLayers.push(tileLayer);
	}

	return tileLayers;
}

//
// GeoJSON 形式ファイルの取得結果からレイヤを作成する
//----------------------------------------------------------------------
//

function createLayer(loaderResult) {

	if (loaderResult[1] !== "success") {
		return null;
	}

	var layer = null;

	try {

		// 以下の場合には null を返却する。
		//
		// - 空のファイルや json として正しくないファイルを読み込んだ場合
		// - 空データを読み込んだ場合
		//     空データ:
		//       {
		//         "type": "FeatureCollection",
		//         "features": []
		//       }

		var data = JSON.parse(loaderResult[0]);
		layer = L.geoJson(data, geoJsonOptions);

		var innerLayers = layer.getLayers();
		if (innerLayers.length === 0) {
			// 空データの場合、表示する図形がないため、レイヤを作成できなかったものとして扱い null を返却する。

			// @Leaflet 0.7.3:
			// - L.geoJson() に空データを渡すと、返却されるオブジェクトには _leaflet_id が振られていない。

			// 本メソッドは、レイヤの表示・非表示のチェックボックスの並び順を考慮して呼び出されるため、
			// もし、空データから生成したオブジェクトを返却してしまうとチェックボックスの並び順がおかしくなるので注意すること。

			layer = null;
		}

	} catch (ex) {
		console.log(ex);
	}

	return layer;
}

//
// すべての GeoJSON 形式ファイルの取得結果からすべてのレイヤを作成する
//----------------------------------------------------------------------
//

function createAllLayer(loaderResultList) {

	// @Leaflet 0.7.3:
	// - レイヤの表示・非表示のチェックボックスの並び順は、Leaflet 内部で振られた _leaflet_id の順となる。

	// このコードでは createLayer() でレイヤが作成された順に _leaflet_id が振られる。
	// チェックボックスの並び順が HZ_layerSelectorControls の指定順になるよう、レイヤを作成する。

	// loaderResultList は HZ_mapLayerDataNames の並びおよび個数と同じ。

	var layers = [];
	var createdLayerDataNames = [];

	for (var i = 0; i < HZ_layerSelectorControls.length; ++i) {
		var selector = HZ_layerSelectorControls[i];

		var layerDataIndex = $.inArray(selector.dataName, HZ_mapLayerDataNames);
		if (layerDataIndex < 0) {
			// 定義間違いに対応するため、念のためチェックする
			continue;
		}

		var loaderResult = loaderResultList[layerDataIndex];
		var layer = createLayer(loaderResult);
		if (layer != null) {
			layers.push({
					dataName: selector.dataName,
					layer: layer
				});
		}

		// createLayer() の成功・失敗に関係無く、再処理させないために、作成済みレイヤとして扱う
		createdLayerDataNames.push(HZ_mapLayerDataNames[layerDataIndex]);
	}


	// 未作成のレイヤ(チェックボックスを表示しないレイヤ)を作成する
	for (var i = 0; i < HZ_mapLayerDataNames.length; ++i) {
		var layerDataName = HZ_mapLayerDataNames[i];

		if (0 <= $.inArray(layerDataName, createdLayerDataNames)) {
			continue;
		}

		var loaderResult = loaderResultList[i];

		var layer = createLayer(loaderResult);
		if (layer != null) {
			layers.push({
					dataName: layerDataName,
					layer: layer
				});
		}
	}

	return layers;
}

//
// マップ上に表示するコントロールをセットアップする
//----------------------------------------------------------------------
//

function setupControls(map, tileLayers, layers) {

	var baseLayers = {};

	for (var i = 0; i < HZ_mapTiles.length; ++i) {
		var tile = HZ_mapTiles[i];

		baseLayers[tile.name] = tileLayers[i];
	}

	var overlays = {};

	for (var i = 0; i < HZ_layerSelectorControls.length; ++i) {
		var selector = HZ_layerSelectorControls[i];

		for (var j = 0; j < layers.length; ++j) {
			if (selector.dataName === layers[j].dataName) {
				overlays[selector.caption] = layers[j].layer;
				break;
			}
		}
	}

	L.control.layers(baseLayers, overlays, {
			position: "topright",
			collapsed: true
		}).addTo(map);
}

//
// layers を HZ_mapLayerDataNames の定義順に並べた一覧を返す
//----------------------------------------------------------------------
//

function getOrderedLayers(layers) {

	var result = [];

	for (var i = 0; i < HZ_mapLayerDataNames.length; ++i) {
		for (var j = 0; j < layers.length; ++j) {
			if (HZ_mapLayerDataNames[i] === layers[j].dataName) {
				result.push(layers[j]);
				break;
			}
		}
	}

	return result;
}

//
// レイヤの重なりを設定する
//----------------------------------------------------------------------
//

function setLayersOrder(layers) {

	for (var i = layers.length - 1; 0 <= i; --i) {
		var layer = layers[i].layer;

		try {
			layer.bringToFront();
		} catch (ex) {
			// @Leaflet 0.7.3:
			// - L.Path.bringToFront(): レイヤ非表示→表示時に
			//   「Uncaught TypeError: Cannot read property '_pathRoot' of null」が起こる場合がある。

			// 当該メソッド内で this._map._pathRoot が存在しない場合は、そもそも処理対象外であるため、
			// 例外をキャッチすることで対応した。
			//console.log("HAZMAP:FIXED:" + ex.message);
		}
	}
}

//
// ファイルを取得する
//----------------------------------------------------------------------
//

function loadGeoJson(url) {

	var defer = new $.Deferred;

	$.ajax({url: url}).done(function() {
		defer.resolve(arguments);
	}).fail(function() {
		// NOTE: ファイルが取得できない場合でも正常終了扱いとする
		////defer.reject(arguments);
		defer.resolve(arguments);
		console.log("HAZMAP:FAILURE:LOAD:" + url);
	});

	return defer.promise();
}

//
// マップとレイヤを表示する
//----------------------------------------------------------------------
//

function displayMapAndLayers() {

	//
	// マップを作成し表示する
	//
	map = L.map("hazmap", {
		center: HZ_mapCenter,
		zoom: HZ_mapInitialZoom,
		minZoom: HZ_mapMinZoom,
		maxZoom: HZ_mapMaxZoom
	});

	// マップの表示領域を設定する
	map.setMaxBounds(HZ_mapMaxBounds);

	// マップにスケールを設定する
	L.control.scale({imperial: false}).addTo(map);


	//
	// マップに表示するタイルレイヤを作成する
	//
	var tileLayers = createAllTileLayer();
	if (0 < tileLayers.length) {
		// 先頭のタイルレイヤを初期表示タイルとする。

		// @Leaflet 0.7.3:
		// - タイルレイヤの選択オプションの並び順は、Leaflet 内部で振られた _leaflet_id の順となる。

		// このコードでは layer.addTo(map) でタイルレイヤがマップに追加された時に _leaflet_id が振られる。
		// 初期表示しないタイルレイヤは作成時には _leaflet_id が振られていないため、
		// 選択オプションの並びは Leaflet の実装に依存する。

		tileLayers[0].addTo(map);
	} else {
		displayError("タイル情報を指定してください。");
		return;
	}

	//
	// マップに表示する GeoJSON 形式ファイルをロードし、作図情報からレイヤを作成・表示する
	//

	var layerDataLoader = [];

	for (var i = 0; i < HZ_mapLayerDataNames.length; ++i) {
		var loader = loadGeoJson(HZ_areaDataDir + "/" + HZ_mapLayerDataNames[i] + ".geojson");
		layerDataLoader.push(loader);
	}

	$.when.apply($, layerDataLoader).done(function() {

		try {

			var layers = createAllLayer(arguments);
			if (0 < layers.length) {

				$.each(layers, function() {
					this.layer.addTo(map);
				});

				setupControls(map, tileLayers, layers);

				var orderLayers = getOrderedLayers(layers);

				setLayersOrder(orderLayers);

				map.on("overlayadd", function(event) {
					setLayersOrder(orderLayers);
				});

			} else {
				displayError("レイヤデータの作成に失敗しました。");
			}

		} catch (ex) {
			displayError("レイヤデータの読込みに失敗しました。");
		}

	}).fail(function() {
		displayError("レイヤデータを取得できません。");
	});
}

//
// スタートアップ
//----------------------------------------------------------------------
//

$(function() {

	// キャプションを設定する
	$("#hazmap-caption").text($("title").text());

	// マップとレイヤを表示する
	displayMapAndLayers();

	// 画面サイズ変更時にマップの大きさを調整する
	$(window).on("load resize", function(event) {

		var windowHeight = $(this).height();
		var captionHeight = $("#hazmap-caption").outerHeight(true);

		// マップの上下を、キャプションのサイズ分空ける
		var mapHeight = windowHeight - (captionHeight * 2);

		$("#hazmap").height(mapHeight);

		if (map != null) {
			map.invalidateSize();
		}
	});

});
