<?php
	/* encoding: UTF-8 & CR+LF */

	$hazmap_version = '1.0.0';

	//
	// REQUEST 値を得る(GET, POST を区別しない)
	//----------------------------------------------------------------------
	//

	function request($name, $defaultValue = '') {
		$result = '';
		if (isset($_REQUEST[$name])) {
			$result = trim($_REQUEST[$name]);
		} else {
			$result = $defaultValue;
		}
		return $result;
	}

	//
	// basename
	//----------------------------------------------------------------------
	//

	function hz_basename($str) {
		$str = basename($str);

		if (preg_match('/[a-zA-Z0-9_-]+/', $str, $matches)) {
			return trim($matches[0]);
		}

		return "";
	}

	//
	// 設定情報を取得する
	//----------------------------------------------------------------------
	//

	function getSetting($settingsJson, $name, $defaultValue) {
		if (isset($settingsJson->$name)) {
			return $settingsJson->$name;
		}

		return $defaultValue;
	}

	//
	// JavaScript での定義を生成する
	//----------------------------------------------------------------------
	//

	function getJsDefinition($definitionItems) {

		$result = '';

		for ($i = 0; $i < count($definitionItems); ++$i) {
			$separator = ($i === 0) ? '' : ', ';

			$item = $definitionItems[$i];

			if (is_string($item)) {

				$result .= sprintf("\t%s\"%s\"\n", $separator, $item);

			} else {
				$result .= sprintf("\t%s{\n", $separator);

				$counter = 0;

				foreach ($item as $key => $value) {
					$subSeparator = ($counter === 0) ? '' : ', ';

					$result .= sprintf("\t\t%s%s: \"%s\"\n", $subSeparator, $key, $value);

					++$counter;
				}

				$result .= "\t}\n";
			}
		}

		return $result;
	}

	//
	// デフォルト設定
	//----------------------------------------------------------------------
	//

	// マップに表示するタイル情報（地図画像）
	$mapTilesDefault = array(
			array(
				'name' => '標準地図',
				'urlTemplate' => 'http://cyberjapandata.gsi.go.jp/xyz/std/{z}/{x}/{y}.png'
			),
		);

	// マップに表示するレイヤデータ名
	$mapLayerDataNamesDefault = array(
			"evacuation_route"	// 避難経路
			, "evacuation_route_arrow"	// 避難経路（進行方向）
			, "temporary_gathering_location"	// 一時避難場所
			, "requires_special_attention_point"	// 要注意箇所
			, "steep_slope_collapse_hazard_area"	// 急傾斜地崩壊危険箇所
			, "pond_hazard_area"	// ため池危険箇所
			, "altitude"	// 標高点
			, "attension_note"	// 注釈等
			, "temporary_gathering_location_label"	// 一時避難場所（ラベル）
			, "requires_special_attention_point_label"	// 要注意箇所（ラベル）
			, "tsunami_inundation_assumption_area"	// 津波浸水想定区域
			, "altitude_label"	// 標高点（ラベル）
			, "water_area"	// 水場
		);

	// マップ上に表示するレイヤの表示・非表示のチェックボックス
	$layerSelectorControlsDefault = array(
			array(
				'caption' => "<img src='images/temporary_gathering_location.png' /> <span>一時避難場所</span>",
				'dataName' => "temporary_gathering_location"
			),
			array(
				'caption' => "<img src='images/temporary_gathering_location_label.png' /> <span>一時避難場所（ラベル）</span>",
				'dataName' => "temporary_gathering_location_label"
			),
			array(
				'caption' => "<img src='images/requires_special_attention_point.png' /> <span>要注意箇所</span>",
				'dataName' => "requires_special_attention_point"
			),
			array(
				'caption' => "<img src='images/requires_special_attention_point_label.png' /> <span>要注意箇所（ラベル）</span>",
				'dataName' => "requires_special_attention_point_label"
			),
			array(
				'caption' => "<img src='images/steep_slope_collapse_hazard_area.png' /> <span>急傾斜地崩壊危険箇所</span>",
				'dataName' => "steep_slope_collapse_hazard_area"
			),
			array(
				'caption' => "<img src='images/pond_hazard_area.png' /> <span>ため池危険箇所</span>",
				'dataName' => "pond_hazard_area"
			),
			array(
				'caption' => "<img src='images/evacuation_route.png' /> <span>避難経路</span>",
				'dataName' => "evacuation_route"
			),
			array(
				'caption' => "<img src='images/evacuation_route_arrow.png' /> <span>避難経路（進行方向）</span>",
				'dataName' => "evacuation_route_arrow"
			),
			array(
				'caption' => "<img src='images/altitude.png' /> <span>標高点</span>",
				'dataName' => "altitude"
			),
			array(
				'caption' => "<img src='images/altitude_label.png' /> <span>標高点（ラベル）</span>",
				'dataName' => "altitude_label"
			),
			array(
				'caption' => "<img src='images/tsunami_inundation_assumption_area.png' /> <span>津波浸水想定区域</span>",
				'dataName' => "tsunami_inundation_assumption_area"
			),
			array(
				'caption' => "<img src='images/water_area.png' /> <span>水場</span>",
				'dataName' => "water_area"
			),
			array(
				'caption' => "<img src='images/attension_note.png' /> <span>注釈等</span>",
				'dataName' => "attension_note"
			)
		);

	// データフォルダ
	$dataDir = 'data';

	// 名称
	$mapName = '津波ハザードマップ';

	// キャプション
	$caption = $mapName;

	// マップに表示するタイル情報（地図画像）の定義
	$mapTiles = array();

	// マップに表示するレイヤデータ名の定義
	$mapLayerDataNames = array();

	// マップ上に表示するレイヤの表示・非表示のチェックボックスの定義
	$layerSelectorControls = array();

	//
	// マップの表示領域
	//

	// 南西
	$mapBoundsSouthWestLat = 20.42527777;	// 緯度
	$mapBoundsSouthWestLng = 122.93361111;	// 経度

	// 北東
	$mapBoundsNorthEastLat = 45.55722222;	// 緯度
	$mapBoundsNorthEastLng = 153.98638888;	// 経度

	//
	// マップの初期表示時の中心位置
	//

	$mapCenterLat = 33.772298;	// 緯度
	$mapCenterLng = 133.516846;	// 経度

	//
	// マップのズームレベル
	//

	$mapMinZoom = 10;	// 最小値
	$mapMaxZoom = 18;	// 最大値
	$mapInitialZoom  = $mapMinZoom;	// 初期値

	//
	// スタートアップ
	//----------------------------------------------------------------------
	//

	// パラメータを取得する
	$area = hz_basename(strtolower(request('area', '')));

	//
	// エリアに対応する設定情報を取得する
	//
	$settingsJsonFilepath = dirname(__FILE__) . "/$dataDir/$area/hazmapSettings.json";
	$settingsJson = null;

	if (file_exists($settingsJsonFilepath)) {
		$contents = file_get_contents($settingsJsonFilepath);
		// メモ帳で編集すると BOM が付く場合がある。
		// BOM 付きだと json_decode() が動作しない場合があるので、除去する
		$contents = preg_replace('/^\xEF\xBB\xBF/', '', $contents);

		$settingsJson = json_decode($contents);
	}

	if ($settingsJson !== null) {

		// 設定情報からマップ設定を取得する
		$areaName = getSetting($settingsJson, 'areaName', '');
		if (!empty($areaName)) {
			$caption = $mapName . ' - ' . $areaName;
		}

		$mapTiles = getSetting($settingsJson, 'mapTiles', $mapTiles);
		$mapLayerDataNames = getSetting($settingsJson, 'mapLayerDataNames', $mapLayerDataNames);
		$layerSelectorControls = getSetting($settingsJson, 'layerSelectorControls', $layerSelectorControls);

		$mapBoundsSouthWestLat = getSetting($settingsJson, 'mapBoundsSouthWestLat', $mapBoundsSouthWestLat);
		$mapBoundsSouthWestLng = getSetting($settingsJson, 'mapBoundsSouthWestLng', $mapBoundsSouthWestLng);
		$mapBoundsNorthEastLat = getSetting($settingsJson, 'mapBoundsNorthEastLat', $mapBoundsNorthEastLat);
		$mapBoundsNorthEastLng = getSetting($settingsJson, 'mapBoundsNorthEastLng', $mapBoundsNorthEastLng);

		$mapCenterLat = getSetting($settingsJson, 'mapCenterLat', $mapCenterLat);
		$mapCenterLng = getSetting($settingsJson, 'mapCenterLng', $mapCenterLng);

		$mapMinZoom = getSetting($settingsJson, 'mapMinZoom', $mapMinZoom);
		$mapMaxZoom = getSetting($settingsJson, 'mapMaxZoom', $mapMaxZoom);
		$mapInitialZoom = getSetting($settingsJson, 'mapInitialZoom', $mapInitialZoom);

	}

	//
	// 定義がない場合のデフォルト値を設定する
	//

	if (count($mapTiles) === 0) {
		$mapTiles = $mapTilesDefault;
	}

	if (count($mapLayerDataNames) === 0) {
		$mapLayerDataNames = $mapLayerDataNamesDefault;
	}

	if (count($layerSelectorControls) === 0) {
		$layerSelectorControls = $layerSelectorControlsDefault;
	}

	//
	// JavaScript での定義を生成する
	//

	$js_mapTiles = getJsDefinition($mapTiles);
	$js_mapLayerDataNames = getJsDefinition($mapLayerDataNames);
	$js_layerSelectorControls = getJsDefinition($layerSelectorControls);

?><!DOCTYPE html>
<html lang="ja">
<head>
	<meta charset="utf-8" />
	<meta http-equiv="X-UA-Compatible" content="IE=edge" />
	<meta name="viewport" content="width=device-width, initial-scale=1" />
	<title><?= $caption ?></title>
	<link rel="stylesheet" href="http://cdn.leafletjs.com/leaflet-0.7.3/leaflet.css" />
	<link rel="stylesheet" href="gsimaps.css" />
	<link rel="stylesheet" href="hazmap.css?v=<?= $hazmap_version ?>" />
</head>
<body>
<div id="hazmap-area">
	<p id="hazmap-caption"><?= $mapName ?></p>
	<div id="hazmap">
		<noscript id="noscript-warn">地図の表示ができません。お使いのブラウザで JavaScript を有効にしてください。</noscript>
		<div id="whileLoadMessage">マップデータをロード中...</div>
		<div id="loadErrorMessage"></div>
	</div>
</div>
<script src="http://code.jquery.com/jquery-1.12.4.min.js" integrity="sha256-ZosEbRLbNQzLpnKIkEdrPv7lOy9C27hHQ+Xp8a4MxAQ=" crossorigin="anonymous"></script>
<script src="http://cdn.leafletjs.com/leaflet-0.7.3/leaflet.js"></script>
<script>

//
// GeoJSON 形式データを格納したフォルダ
//----------------------------------------------------------------------
//

var HZ_areaDataDir = "<?= $dataDir ?>/<?= $area ?>";

//
// マップに表示するタイル情報（地図画像）の定義
//----------------------------------------------------------------------
// 定義の先頭のものが初期表示タイルとなる。
//

var HZ_mapTiles = [
<?= $js_mapTiles ?>
];

//
// マップに表示するレイヤデータ名の定義
//----------------------------------------------------------------------
// 名称の後ろに拡張子 ".geojson" を追加した名称がファイル名となる。
// ファイルは HZ_areaDataDir で指定したディレクトリ下に格納しておくこと。
// また、ここでの定義順がレイヤの上からの重なり順となるよう実装しているため、
// 希望する表示時のレイヤの重なりにあわせて、上から下へと定義すること。
//

var HZ_mapLayerDataNames = [
<?= $js_mapLayerDataNames ?>
];

//
// マップ上に表示するレイヤの表示・非表示のチェックボックスの定義
//----------------------------------------------------------------------
// 希望する表示順にあわせて、上から下へと定義すること。
// ここに定義しないレイヤデータ名に対応するレイヤについては
// チェックボックスを表示しない(レイヤは常に表示され非表示にできない)。
// なお、dataName は HZ_mapLayerDataNames で定義した値を指定すること。
//

var HZ_layerSelectorControls = [
<?= $js_layerSelectorControls ?>
];

//
// マップの表示領域
//----------------------------------------------------------------------
// マップの表示領域を制限する。
// 必ず矩形領域が指定されるよう注意すること。
//
// [[南西緯度, 南西経度], [北東緯度, 北東経度]]
//

var HZ_mapMaxBounds = [[<?= $mapBoundsSouthWestLat ?>, <?= $mapBoundsSouthWestLng ?>], [<?= $mapBoundsNorthEastLat ?>, <?= $mapBoundsNorthEastLng ?>]];

//
// マップの初期表示時の中心位置
//----------------------------------------------------------------------
// 必ず HZ_mapMaxBounds 内の位置を指定すること。
//
// [緯度, 経度]
//

var HZ_mapCenter = [<?= $mapCenterLat ?>, <?= $mapCenterLng ?>];

//
// マップのズームレベルの最小値
//----------------------------------------------------------------------
// 使用するタイル（地図画像）によって変わるため、実際に表示して調整すること。
// 必ず HZ_mapMinZoom <= HZ_mapMaxZoom となるよう注意すること。
//

var HZ_mapMinZoom = <?= $mapMinZoom ?>;

//
// マップのズームレベルの最大値
//----------------------------------------------------------------------
// 使用するタイル（地図画像）によって変わるため、実際に表示して調整すること。
// 必ず HZ_mapMinZoom <= HZ_mapMaxZoom となるよう注意すること。
//

var HZ_mapMaxZoom = <?= $mapMaxZoom ?>;

//
// マップのズームレベルの初期値
//----------------------------------------------------------------------
// 必ず HZ_mapMinZoom <= HZ_mapInitialZoom <= HZ_mapMaxZoom となるよう注意すること。
//

var HZ_mapInitialZoom = <?= $mapInitialZoom ?>;

</script>
<script src="hazmap.js?v=<?= $hazmap_version ?>"></script>
</body>
</html>
